from opentelemetry.instrumentation.distro import BaseDistro

from .uptrace import configure_opentelemetry


# pylint: disable=too-few-public-methods
class UptraceDistro(BaseDistro):
    def _configure(self, **kwargs):
        configure_opentelemetry()



import logging

import grpc
from opentelemetry._logs import set_logger_provider
from opentelemetry.exporter.otlp.proto.grpc._log_exporter import (
    OTLPLogExporter,
)
from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from opentelemetry.sdk.resources import Resource

from .dsn import DSN


def configure_logs(dsn: DSN, resource: Resource, level=logging.NOTSET):
    logger_provider = LoggerProvider(resource=resource)
    set_logger_provider(logger_provider)

    exporter = OTLPLogExporter(
        endpoint=dsn.otlp_grpc_addr,
        headers=(("uptrace-dsn", dsn.str),),
        timeout=5,
        compression=grpc.Compression.Gzip,
    )
    logger_provider.add_log_record_processor(BatchLogRecordProcessor(exporter))

    handler = LoggingHandler(level=level, logger_provider=logger_provider)
    logging.getLogger().addHandler(handler)



import grpc
from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor

from .dsn import DSN
from .id_generator import UptraceIdGenerator


def configure_traces(
    dsn: DSN,
    resource: Resource,
):
    provider = TracerProvider(resource=resource, id_generator=UptraceIdGenerator())
    trace.set_tracer_provider(provider)

    exporter = OTLPSpanExporter(
        endpoint=dsn.otlp_grpc_addr,
        headers=(("uptrace-dsn", dsn.str),),
        timeout=5,
        compression=grpc.Compression.Gzip,
    )

    bsp = BatchSpanProcessor(
        exporter,
        max_queue_size=1000,
        max_export_batch_size=1000,
        schedule_delay_millis=5000,
    )
    trace.get_tracer_provider().add_span_processor(bsp)



from urllib.parse import urlparse

from .util import remove_prefix


class DSN:
    # pylint:disable=too-many-arguments
    def __init__(self, dsn="", scheme="", host="", port="", project_id="", token=""):
        self.str = dsn
        self.scheme = scheme
        self.host = host
        self.port = port
        self.project_id = project_id
        self.token = token

    def __str__(self):
        return self.str

    @property
    def app_addr(self):
        if self.host == "uptrace.dev":
            return f"{self.scheme}://app.uptrace.dev"
        return f"{self.scheme}://{self.host}:14318"

    @property
    def otlp_http_addr(self):
        if self.host == "uptrace.dev":
            return "https://otlp.uptrace.dev"
        return f"{self.scheme}://{self.host}:{self.port}"

    @property
    def otlp_grpc_addr(self):
        if self.host == "uptrace.dev":
            return "https://otlp.uptrace.dev:4317"
        return f"{self.scheme}://{self.host}:{self.port}"


def parse_dsn(dsn: str) -> DSN:
    if dsn == "":
        raise ValueError("either dsn option or UPTRACE_DSN is required")

    o = urlparse(dsn)
    if not o.scheme:
        raise ValueError(f"can't parse DSN={dsn}")

    host = o.hostname
    if not host:
        raise ValueError(f"DSN={dsn} does not have a host")
    if host == "api.uptrace.dev":
        host = "uptrace.dev"

    if host != "uptrace.dev":
        return DSN(dsn=dsn, scheme=o.scheme, host=host, port=o.port)

    project_id = remove_prefix(o.path, "/")
    token = o.username

    return DSN(
        dsn=dsn,
        scheme=o.scheme,
        host=host,
        port=o.port,
        project_id=project_id,
        token=token,
    )



"""Uptrace exporter for OpenTelemetry"""

from .dsn import parse_dsn
from .uptrace import (
    configure_opentelemetry,
    force_flush,
    report_exception,
    shutdown,
    trace_url,
)
from .version import __version__

__all__ = [
    "configure_opentelemetry",
    "force_flush",
    "shutdown",
    "trace_url",
    "report_exception",
    "parse_dsn",
    "__version__",
]



import grpc
from opentelemetry import metrics
from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import (
    OTLPMetricExporter,
)
from opentelemetry.sdk import metrics as sdkmetrics
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    AggregationTemporality,
    PeriodicExportingMetricReader,
)
from opentelemetry.sdk.metrics.view import ExponentialBucketHistogramAggregation
from opentelemetry.sdk.resources import Resource

from .dsn import DSN

temporality_delta = {
    sdkmetrics.Counter: AggregationTemporality.DELTA,
    sdkmetrics.UpDownCounter: AggregationTemporality.DELTA,
    sdkmetrics.Histogram: AggregationTemporality.DELTA,
    sdkmetrics.ObservableCounter: AggregationTemporality.DELTA,
    sdkmetrics.ObservableUpDownCounter: AggregationTemporality.DELTA,
    sdkmetrics.ObservableGauge: AggregationTemporality.DELTA,
}


def configure_metrics(
    dsn: DSN,
    resource: Resource,
):
    exporter = OTLPMetricExporter(
        endpoint=dsn.otlp_grpc_addr,
        headers=(("uptrace-dsn", dsn.str),),
        timeout=5,
        compression=grpc.Compression.Gzip,
        preferred_temporality=temporality_delta,
        preferred_aggregation={
            sdkmetrics.Histogram: ExponentialBucketHistogramAggregation()
        },
    )
    reader = PeriodicExportingMetricReader(exporter)
    provider = MeterProvider(metric_readers=[reader], resource=resource)
    metrics.set_meter_provider(provider)
